/*----------------------------------------------------------------------------

	example2.cpp
	Copyright 1997 Sumus Computing Incorporated, All Rights Reserved.
	
----------------------------------------------------------------------------*/
// Tree view example 2 (volume directory tree)

#include "ex_appl.h"

enum e8BitColors
{
	cWW = 0xFF,	// white
	cBB = 0x00,	// black
	cRR = 0x2A,	// red

	cDB = 0x25,	// dark blue
	cLB = 0x66,	// light blue

	cDG = 0x36,	// dark gray
	cLG = 0x46,	// light gray

	cYY = 0xE5	// yellow
};


static unsigned char const dataBITMAP_FOLDER[] = 
{
	9,7,
	cWW,cBB,cBB,cBB,cWW,cWW,cWW,cWW,cWW,0,0,0,
	cBB,cYY,cYY,cYY,cBB,cWW,cWW,cWW,cWW,0,0,0,
	cBB,cYY,cYY,cYY,cYY,cBB,cBB,cBB,cBB,0,0,0,
	cBB,cYY,cYY,cYY,cYY,cYY,cYY,cYY,cBB,0,0,0,
	cBB,cYY,cYY,cYY,cYY,cYY,cYY,cYY,cBB,0,0,0,
	cBB,cYY,cYY,cYY,cYY,cYY,cYY,cYY,cBB,0,0,0,
	cBB,cBB,cBB,cBB,cBB,cBB,cBB,cBB,cBB,0,0,0
};


//----------------------------------------------------------------------------

class ExampleApplication: public AApplication 
{
public:
	ExampleApplication();
	~ExampleApplication();

	virtual void  AddToTreeView();
	static BBitmap const *bitmapFolder;
};


BBitmap const* ExampleApplication::bitmapFolder = NULL;


//----------------------------------------------------------------------------

class AStoreTreeItem: public ATreeItem
{
public:
	enum eItemType { itemROOT, itemFOLDER, itemFILE};
	
	AStoreTreeItem(char const* a_string,eItemType a_type,record_ref a_ref);
	~AStoreTreeItem();

	virtual char const* GetLabel() const;
	virtual BBitmap const* GetIcon() const;

	static void FindIconTable(BVolume& volume);

	virtual bool CanBeCollapsed() const;
	virtual bool CanBeExpanded() const;

	virtual void Collapse();
	virtual void Expand();

	bool OpenFolder();
	
protected:
	char		*string;
	eItemType	itemType;
	record_ref	recordRef;
	BBitmap		*bitmap;

	static BTable		*iconTable;
	static BDatabase	*db;
};


BTable*		AStoreTreeItem::iconTable	= NULL;
BDatabase*	AStoreTreeItem::db			= NULL;


//----------------------------------------------------------------------------

ExampleApplication::ExampleApplication()
  : AApplication(18)
{
	bitmapFolder = g_NewBitmap(dataBITMAP_FOLDER);
}


ExampleApplication::~ExampleApplication()
{
	delete bitmapFolder;
}


void ExampleApplication::AddToTreeView()
{
	BVolume volume = volume_at(0);
	if(B_ERROR==volume.Error())
		return;
		
	BDirectory rootDir;
	volume.GetRootDirectory(&rootDir);

	char rootDirName[B_FILE_NAME_LENGTH];
	rootDir.GetName(rootDirName);

	AStoreTreeItem::FindIconTable(volume);

	AStoreTreeItem *item =
		new AStoreTreeItem(	rootDirName,AStoreTreeItem::itemFOLDER,
							rootDir.Record()->Ref());
		
	item->Collapse();
	
	treeView->Root().AddItem(*item);
}


//----------------------------------------------------------------------------

AStoreTreeItem::AStoreTreeItem(	char const* a_string,eItemType a_type,
								record_ref a_ref)
  :	ATreeItem(), string(NULL), itemType(a_type),
  	recordRef(a_ref), bitmap(NULL)
{
	string = strdup(a_string ? a_string : "");

	if(itemFILE==itemType)
	{
		// load the bitmap
		if(iconTable)
		{
			BFile file;
			file.SetRef(recordRef);
			
			ulong type,creator;
			if(B_NO_ERROR==file.GetTypeAndApp(&type,&creator))
			{
				BQuery *query = new BQuery();
				query->AddTable(iconTable);
				
				query->PushField("creator"); 
				query->PushLong(creator);
				query->PushOp(B_EQ);
				
				query->PushField("type"); 
				query->PushLong(type);
				query->PushOp(B_EQ);

				query->PushOp(B_AND);
			
				// note: FetchOne() may return an empty set of records
				if(B_NO_ERROR==query->FetchOne() && 0<query->CountRecordIDs())
				{
					record_id rid = query->RecordIDAt(0);
					BRecord *record = new BRecord(db,rid);
					
					long rawSize;
					void *raw = record->FindRaw("smallBits",&rawSize);
					if(B_NO_ERROR==record->Error() && 256==rawSize)
					{
						bitmap = g_NewBitmap(16,16,raw);
					}
					
					delete record;
				}
					
				delete query;
			}
		}
	}
}


AStoreTreeItem::~AStoreTreeItem()
{
	delete bitmap;
	delete string;
}


inline char const* AStoreTreeItem::GetLabel() const
{
	return string;
}


BBitmap const* AStoreTreeItem::GetIcon() const
{
	// if the bitmap is loaded
	if(bitmap) return bitmap;

	// else get some default	
	switch(itemType)
	{
	case itemFOLDER:
		return ExampleApplication::bitmapFolder;
	default:
		return inherited::GetIcon();
	}
}


void AStoreTreeItem::FindIconTable(BVolume& volume)
{
	db = volume.Database();
	iconTable = db->FindTable("Icon");	
}


bool AStoreTreeItem::CanBeCollapsed() const
{
	return inherited::CanBeCollapsed();
}


bool AStoreTreeItem::CanBeExpanded() const
{
	return itemFOLDER==itemType && IsCollapsed();
}


void AStoreTreeItem::Collapse()
{
	inherited::Collapse();	// collapse ..
	MakeEmptyAll(true);		// .. and destroy the invisible items
}


void AStoreTreeItem::Expand()
{
	OpenFolder();			// reload the folder content
	inherited::Expand();	// .. and make it visible
}

	
bool AStoreTreeItem::OpenFolder()
{	
	if(itemFOLDER!=itemType) return false;

	BDirectory dir;
	dir.SetRef(recordRef);

	long fileCount = dir.CountFiles();
	long subdCount = dir.CountDirectories();

	record_ref *refv = new record_ref[max(fileCount,subdCount)];

	dir.GetFiles(0,fileCount,refv);
	for(long l=0; l<fileCount; l++)	// all files
	{
		BFile file;
		file.SetRef(refv[l]);
		
		char fileName[B_FILE_NAME_LENGTH];
		file.GetName(fileName);

		ATreeItem *fileItem = new AStoreTreeItem(fileName,AStoreTreeItem::itemFILE,refv[l]);
		fileItem->Collapse();		// flagged as expanded by default
		this->AddItem(*fileItem);
	}
	
	dir.GetDirectories(0,subdCount,refv);
	for(long l=0; l<subdCount; l++)	// all subdirectories
	{
		BDirectory subd;
		subd.SetRef(refv[l]);
		
		char dirName[B_FILE_NAME_LENGTH];
		subd.GetName(dirName);

		ATreeItem *dirItem = new AStoreTreeItem(dirName,AStoreTreeItem::itemFOLDER,refv[l]);
		dirItem->Collapse();		// flagged as expanded by default
		this->AddItem(*dirItem);
	}
	
	delete [] refv;

	return true;
}


//----------------------------------------------------------------------------


int main(void)
{
	return ExampleApplication().Run();	
}


//-- eof ---------------------------------------------------------------------
